package com.examples;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class DAOBase 
{
	protected Connection   connection = null;

	private   final  String  INSERT_blob   = "insert into %s(%s) values(%d)"    ;
	private   final  String  UPDATE_blob   = "update %s set %s = ? where %s = ?";
	private   final  String  SELECT_blob   = "select %s from %s where %s = %d"  ;
	
	protected final  String  INSERT_clob   = "insert into %s(%s) values(%d)"    ;
	protected final  String  UPDATE_clob   = "update %s set %s = ? where %s = ?";
	private   final  String  SELECT_clob   = "select %s from %s where id = %d"  ;
	
	protected final  String  TABLE_DROP    = "DROP TABLE %s";
	
	private   final  String  CHARSET_UTF8  = "UTF-8";
	protected final  int     BUFFER_length = 2 * 1024;
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void createConnection(){}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public Connection getConnection()
	{
		return connection;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void closeConnection() 
	{
		if(connection != null){
			try {
				connection.close();
			}
			catch(SQLException e){
				System.out.println(e.getMessage());
			}
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
     * Функция выполнения SQL-запроса
     * @param sql текст запроса
     * @return результат выполнения запроса
     */
    public boolean execSQL (final String sql) {
        boolean result = false;
        try {
            if (connection != null) {
                Statement statement = connection.createStatement();
                statement.execute(sql);
                statement.close();
                statement = null;
                result = true;
            }
        } catch (SQLException e) {
            System.err.println ("SQLException : code = " + 
                                String.valueOf(e.getErrorCode()) 
                                     + " - " + e.getMessage());
            System.err.println ("\tSQL : " + sql);
	        
        }
        return result;
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public boolean createTable(final String sql)
	{
		return execSQL (sql);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public boolean dropTable(final String table)
	{
		return execSQL (String.format(TABLE_DROP, table));
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи файла в поле BLOB
	 * @param table наименование таблицы
	 * @param field наименование поля BLOB в таблице
	 * @param pk Primary Key
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return результат записи (true - OK, false - failed)
	 */
	public boolean writeBlob(final String table, final String field, 
			                 final String pk, final int id, 
			                 final String fpath)
	{
		boolean result = execSQL (String.format(INSERT_blob, table, pk, id));
		if (result) {
			PreparedStatement ps = null;
	        File file = new File(fpath);
	        try {
	        	String sql = String.format(UPDATE_blob, table, field, pk);
				FileInputStream is = new FileInputStream(file);
				ps = connection.prepareStatement(sql);
				ps.setBinaryStream(1, is, (int)file.length());
				ps.setInt(2, id);
				ps.executeUpdate();

				connection.commit();
				ps.close();
			} catch (FileNotFoundException e) {
				result = false;
				e.printStackTrace();
			} catch (SQLException e) {
				result = false;
				e.printStackTrace();
			}
		}
		return result;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция чтения поля BLOB
	 * @param id идентифкатор записи
	 * @return поле BLOB
	 * @throws SQLException
	 */
	private Blob readBlobField(final String table, final String field, 
                               final String pk, final int id) 
                               throws SQLException 
	{
		String sql = String.format(SELECT_blob, field, table, pk, id);
		Statement stmt = connection.createStatement();
		ResultSet rs = stmt.executeQuery(sql); 
		Blob blob = null; 
		if (rs.next()) 
			blob = rs.getBlob(1); 
		return blob; 
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи значения поля BLOB в поток
	 * @param blob поле BLOB
	 * @param out выходной поток
	 * @return размер поля BLOB
	 * @throws SQLException
	 * @throws IOException
	 */
	private long writeFromBlob2Stream(Blob blob, OutputStream out) throws SQLException, IOException
	{ 
		InputStream is = blob.getBinaryStream(); 
		int length = -1; 
		long size = 0;
		byte[] buf = new byte[BUFFER_length]; 
		while ((length = is.read(buf)) != -1) { 
			out.write(buf, 0, length); 
			size += length; 
		} 
		is.close(); 
		return size; 
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи файла в поле BLOB
	 * @param table наименование таблицы
	 * @param field наименование поля BLOB в таблице
	 * @param pk Primary Key
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return результат записи (true - OK, false - failed)
	 */
	public long readBlobToFile(final String table, final String field, 
                               final String pk, final int id, final String fpath) 
           		               throws IOException, SQLException
	{ 
		long size = 0; 
		OutputStream fwriter = new FileOutputStream(fpath);
		Blob blob =  readBlobField(table, field, pk, id);
		size = writeFromBlob2Stream(blob, fwriter); 
		fwriter.close(); 
		return size; 
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи содержимого файла в поле CLOB
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return логическое значение, результат выполнения опреации
	 */
	public boolean writeClob(final int id, final String fpath)
	{
		return false;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функцяя чтения поля CLOB
	 * @param table таблица
	 * @param field поле CLOB в таблице
	 * @param id идентификатор записи
	 * @return поле CLOB
	 * @throws SQLException
	 */
	private Clob readClobField(final String table, final String field, final int id) throws SQLException
	{ 
		String sql = String.format(SELECT_clob, field, table, id);
		Statement stmt = connection.createStatement(); 
		ResultSet rs = stmt.executeQuery(sql); 
		Clob clob = null; 
		if (rs.next()) 
			clob = rs.getClob(1); 
		return clob; 
	} 
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция чтения значения поля CLOB
	 * @param clob объект CLOB
	 * @param out выходной поток
	 * @return размер
	 * @throws SQLException
	 * @throws IOException
	 */
	private long readFromClob2Stream(Clob clob, Writer out) throws SQLException, IOException
	{ 
		BufferedReader breader = new BufferedReader(clob.getCharacterStream()); 
		int length = -1; 
		long size = 0; 
		char[] buf = new char[BUFFER_length]; 
		while ((length = breader.read(buf, 0, BUFFER_length)) != -1) { 
			out.write(buf, 0, length); 
			size += length; 
		} 
		breader.close(); 
		return size; 
	} 
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция выгрузки CLOB объекта в файл
	 * @param table таблица
	 * @param field поле CLOB в таблице
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return размер содержимого файла
	 * @throws IOException
	 * @throws SQLException
	 */
	public long readClobToFile(final String table, final String field, 
			                   final int id, final String fpath) 
			               	   throws IOException, SQLException
	{
		long size = 0; 
		BufferedWriter fwriter = new BufferedWriter(new FileWriter(fpath));
		Clob clob = readClobField (table, field, id);
		size = readFromClob2Stream(clob, fwriter); 
		fwriter.close(); 
		return size; 
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция чтения поля CLOB 
	 * @param table таблица
	 * @param field поле CLOB в таблице
	 * @param id идентификатор записи
	 * @return значение поля CLOB
	 */
	public String readClobData(final String table, final String field, final int id)
	{
		StringBuffer buffer = new StringBuffer();
		try {
			Clob clob = readClobField (table, field, id);
			
			BufferedReader reader = new BufferedReader(clob.getCharacterStream());
			char[] buf = new char[BUFFER_length];
			int length = -1; 
			try {
				while ((length = reader.read(buf, 0, BUFFER_length)) != -1) {
					if (length == BUFFER_length)
						buffer.append(String.valueOf(buf));
					else
						buffer.append(String.valueOf(buf).substring(0, length));
				}
				reader.close(); 
			} catch (IOException e) {
				e.printStackTrace();
			} 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return buffer.toString();
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Функция записи содержимого файла в поле CLOB 
	 * @param table наименование таблицы
	 * @param field наименование поля BLOB в таблице
	 * @param pk Primary Key
	 * @param id идентификатор записи
	 * @param fpath путь к файлу
	 * @return результат записи (true - OK, false - failed)
	 */
	public boolean writeClob(final String table, final String field, 
                             final String pk, final int id,
                             final String fpath)
	{
		String sql = String.format(INSERT_clob, table, pk, id);
		boolean result = execSQL (sql);
		if (result) {
			PreparedStatement ps = null;
	        File file = new File(fpath);

	        try {
				FileInputStream   fis = new FileInputStream  (file);
				InputStreamReader isr = new InputStreamReader(fis, CHARSET_UTF8);
		        BufferedReader    br  = new BufferedReader   (isr);
		        
		        sql = String.format(UPDATE_clob, table, field, pk);
				ps = connection.prepareStatement(sql);
				ps.setCharacterStream(1, br, (int)file.length());
				ps.setInt(2, id);
				ps.executeUpdate();

				connection.commit();
				ps.close();
			} catch (UnsupportedEncodingException e) {
	            result = false;
				e.printStackTrace();
			} catch (FileNotFoundException e) {
				result = false;
				e.printStackTrace();
			} catch (SQLException e) {
				result = false;
				e.printStackTrace();
			}
		}
		return result;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
