package com.examples;

import java.io.IOException;
import java.sql.SQLException;

public class MainTest
{
	private boolean TEST_mysql  = false;
	private boolean TEST_oracle = true;
	
	private final String TEXT_tbl_create = "\nTable <%s> created";
	private final String TEXT_tbl_insert = "%s inserted into table";
	private final String TEXT_tbl_upload = "%s (size %d byte) uploaded";
	private final String TEXT_tbl_drop   = "Table <%s> dropped";
	
	public MainTest()
	{
		if (TEST_oracle)
			testOracle();
		if (TEST_mysql)
			testMySQL();
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void testDAO(DAOBase dao, final String caption)
	{
		String text;
		if (dao.getConnection() != null) {
			System.out.println(caption);
			System.out.println("Connection to server");

			if (dao.createTable("blobs")) {
				text = String.format(TEXT_tbl_create, "blobs");
				System.out.println(text);
				
				if (dao.writeBlob("blobs", "data", "id", 1, "aircraft.jpg")) {
					text = String.format(TEXT_tbl_insert, "Image");
					System.out.println(text);
					try {
						long size = dao.readBlobToFile("blobs", "data", "id", 1, "aircraft1.jpg");
						text = String.format(TEXT_tbl_upload, "Image", size);
						System.out.println(text);
					} catch (IOException e) {
					} catch (SQLException e) {}
				}
				if (dao.writeBlob("blobs", "data", "id", 2, "отчет.xlsx")) {
					text = String.format(TEXT_tbl_insert, "XLSX");
					System.out.println(text);
					try {
						long size = dao.readBlobToFile("blobs", "data", "id", 2, "отчет1.xlsx");
						text = String.format(TEXT_tbl_upload, "XLSX", size);
						System.out.println(text);
					} catch (IOException e) {
					} catch (SQLException e) {}
				}
				
				if (dao.dropTable("blobs")) {
					text = String.format(TEXT_tbl_drop, "blobs");
					System.out.println(text);
				}
			}			
			if (dao.createTable("files")) {
				text = String.format(TEXT_tbl_create, "files");
				System.out.println(text);
				if (dao.writeClob("files", "data", "id", 1, "текст.txt")) {
					text = String.format(TEXT_tbl_insert, "File");
					System.out.println(text);
					try {
						// Запись значения поля CLOB в файл
						long size = dao.readClobToFile("files", "data", 1, "текст1.txt");
						text = String.format(TEXT_tbl_upload, "File", size);
						System.out.println(text);
						// Запись значения поля CLOB в переменную
						String content = dao.readClobData("files", "data", 1);
						System.out.println("\nCLOB data (length = " + content.length() + ") :\n" + content);
					} catch (IOException e) {
					} catch (SQLException e) {}
				}
				if (dao.dropTable("files")) {
					text = String.format(TEXT_tbl_drop, "files");
					System.out.println(text);
				}
			}			
			dao.closeConnection();
		} else
			System.out.println("Connection is NULL");
	}
	private void testMySQL()
	{
		DAOBase dao = new MySQLDAO();
		testDAO(dao, "----- TEST MySQL -----\n");
	}
	private void testOracle()
	{
		DAOBase dao = new OracleDAO();
		testDAO(dao, "----- TEST Oracle -----\n");
	}
	public static void main(String[] args)
	{
		new MainTest();
		System.exit(0);
	}
}
